#ifndef QTOGREWINDOW_H
#define QTOGREWINDOW_H

/*
Source: http://www.ogre3d.org/tikiwiki/tiki-index.php?page=Integrating+Ogre+into+QT5

For the purposes of creating a bare-bones Qt implementation I found that I only need to include these three (3) Qt headers.
*/
#include <QtWidgets/QApplication>
#include <QtGui/QKeyEvent>
#include <QtGui/QWindow>

/*
One more for time measurement
*/
#include <QElapsedTimer>

/*
As expected; the Ogre3D header.
*/
#include <Ogre.h>

/*
Changed SdkCameraMan implementation to work with QKeyEvent, QMouseEvent, QWheelEvent
*/
#include "SdkQtCameraMan.h"

/*
With the headers included we now need to inherit from QWindow.
*/
class QTOgreWindow : public QWindow
{
	/*
	A QWindow still inherits from QObject and can have signals/slots; we need to add the appropriate Q_OBJECT keyword so that Qt's intermediate compiler can do the necessary wireup between our class and the rest of Qt.
	*/

	Q_OBJECT

public:
	explicit QTOgreWindow(QWindow *parent = NULL);
	~QTOgreWindow();

	/*
	We declare these methods virtual to allow for further inheritance.
	*/
	virtual void render(QPainter *painter);
	virtual void render();
	virtual void initialize();
	virtual void createScene();
#if OGRE_VERSION >= ((2 << 16) | (0 << 8) | 0)
	virtual void createCompositor();
#endif

	void setAnimating(bool animating);
public slots:

	virtual void renderLater();
	virtual void renderNow();
    virtual void update();

	/*
	We use an event filter to be able to capture keyboard/mouse events. More on this later.
	*/
	virtual bool eventFilter(QObject *target, QEvent *event);

protected:
	/*
	Ogre3D pointers added here. Useful to have the pointers here for use by the window later.
	*/
	Ogre::Root* m_ogreRoot;
	Ogre::RenderWindow* m_ogreWindow;
	Ogre::SceneManager* m_ogreSceneMgr;
	Ogre::Camera* m_ogreCamera;
	Ogre::ColourValue m_ogreBackground;
    OgreQtBites::SdkQtCameraMan* m_cameraMan;

    QElapsedTimer m_elapsedTimer;

	bool m_update_pending;
	bool m_animating;

	/*
	The below methods are what is actually fired when they keys on the keyboard are hit.
	Similar events are fired when the mouse is pressed or other events occur.
	*/
	virtual void keyPressEvent(QKeyEvent * ev);
	virtual void keyReleaseEvent(QKeyEvent * ev);
    virtual void mouseMoveEvent(QMouseEvent* e);
    virtual void mouseWheelEvent(QWheelEvent* e);
    virtual void mousePressEvent(QMouseEvent* e);
    virtual void mouseReleaseEvent(QMouseEvent* e);
	virtual void exposeEvent(QExposeEvent *event);
	virtual bool event(QEvent *event);

	void log(Ogre::String msg);
	void log(QString msg);
};

#endif // QTOGREWINDOW_H
